import network, rp2, time
import socket
import umsgpack
from machine import Pin

led = Pin("LED", Pin.OUT)
led.on()

# set your WiFi Country
rp2.country('US')

wlan = network.WLAN(network.STA_IF)
wlan.active(True)

# set power mode to get WiFi power-saving off (if needed)
wlan.config(pm = 0xa11140)

wlan.connect('A-Net', 'Sam2Curly')


while not wlan.isconnected() and wlan.status() >= 0:
 print("Waiting to connect:")
 time.sleep(1)

print(wlan.ifconfig())

# create a socket server
serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
# serversocket.setblocking(False)

addr = ("",31335)
serversocket.bind(addr)
serversocket.listen()
(clientsocket, address) = serversocket.accept()
print((clientsocket, address))
led.off()

count = 0
while True:
    payload = {'pico_data': count}
    count += 1
    packed = umsgpack.dumps(payload)
    clientsocket.sendall(packed)
    time.sleep(.1)
    print(f'sending {count}')
    
    






packed = bytearray(b'\x81\xa9pico_data\x00')
p_length = bytearray(len(packed).to_bytes(1, 'big'))


# packed = bytearray(umsgpack.dumps(payload))

# get the length of the packed bytearray and convert that to a
# byte array


# append the length to the packed bytarray
p_length.extend(packed)
# packed = p_length.extend(packed)
packed = bytes(p_length)
print(packed)
# ly = bytearray(len(b).to_bytes(1, 'big'))
# ly.extend(bytearray(b))
# m = bytes(ly)
# print(m)



"""
tcp_gateway.py

 Copyright (c) 2022 Alan Yorinks All right reserved.

 The tcp_gateway is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import argparse
import asyncio
import json
import signal
import sys
import zmq

from python_banyan.banyan_base_aio import BanyanBaseAIO
from python_banyan.utils.tcp_gateway.tcp_socket import TcpSocket


class TcpGateWay(BanyanBaseAIO):
    """
    This class is a gateway between a python-banyan network and a TCP socket.

    The TcpGateway serves as a simple connector between a python-banyan network and a TCP
    server. It acts as a TCP client and connects to a TCP server.
    The connected TCP server must have the capability to both MessagePack encode and
    decode messages. The TcpGateway does not provide any encoding or decoding of its own.
    """

    # noinspection PyUnusedLocal
    def __init__(self, tcp_server_ip_address=None, tcp_server_ip_port=31335,
                 subscriber_list=None, banyan_pub_topic=None, back_plane_ip_address=None,
                 subscriber_port='43125', publisher_port='43124',
                 process_name='TCPGateWay', event_loop=None, auto_start=True):
        """
        :param tcp_server_ip_address: TCP IP address of server in string format
        :param tcp_server_ip_port: TCP IP port of server in numerical format
        :param subscriber_list: A list of banyan topics
        :param banyan_pub_topic: published message topic
        :param back_plane_ip_address: Banyan backplane IP address - will be auto-detected
        :param subscriber_port: Banyan subscriber port
        :param publisher_port: Banyan publisher port
        :param process_name: TCP Gateway
        :param event_loop: asyncio event loop
        :param auto_start: automatically start the tasks
        """
        # initialize the base class
        super(TcpGateWay, self).__init__(back_plane_ip_address, subscriber_port,
                                          publisher_port, process_name=process_name)

        if not tcp_server_ip_address:
            raise RuntimeError("A TCP IP address for the TCP server must be provided.")

        self.tcp_server_ip_address = tcp_server_ip_address

        self.tcp_server_ip_port = tcp_server_ip_port

        # save the publication topic
        if not banyan_pub_topic:
            raise RuntimeError('A publishing topic must be provided')

        # encode the topic
        self.banyan_pub_topic = banyan_pub_topic.encode()

        if not subscriber_list:
            raise RuntimeError('A list of subscription topics must be provided')

        self.subscriber_list = subscriber_list

        if not self.back_plane_ip_address:
            self.back_plane_ip_address = back_plane_ip_address

        self.subscriber_port = subscriber_port

        self.publisher_port = publisher_port

        # if the user passes in a single topic, convert the topic to a list of topics
        if type(self.subscriber_list) is not list:
            self.subscriber_list = [self.subscriber_list]

        if not event_loop:
            self.event_loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.event_loop)

        # tcp client socket
        self.sock = None

        self.tcp_recv_task = None

        if auto_start:
            self.event_loop.run_until_complete(self.start_aio())

    async def start_aio(self):
        """
        Start up the tasks
        """

        await self.begin()

        self.sock = TcpSocket(self.tcp_server_ip_address, self.tcp_server_ip_port,
                              self.event_loop)
        await self.sock.start()

        self.the_task = self.event_loop.create_task(self._receive_tcp_data())

        while True:
            await asyncio.sleep(.1)

    async def _receive_tcp_data(self):
        while True:
            # get the length of the next packet
            byte_val = await self.sock.read(1)
            length = int.from_bytes(byte_val, "big")

            pico_packet = await self.sock.read(length)

            await self.publisher.send_multipart([self.banyan_pub_topic, pico_packet])

    async def receive_loop(self):
        """
        This is the receive loop for Banyan messages.

        This method may be overwritten to meet the needs
        of the application before handling received messages.

        """
        while True:
            data = await self.subscriber.recv_multipart()
            msg = data[1]
            # p_length = len(msg)
            # msg = await self.unpack(data[1])
            # get the length of the payload and express as a bytearray
            p_length = bytearray(len(msg).to_bytes(1, 'big'))

            # append the length to the packed bytarray
            p_length.extend(msg)

            # convert from bytearray to bytes
            msg = bytes(p_length)
            # print(f'Message from PC host: {msg}')
            await self.sock.write(msg)


def tcp_gateway():
    # allow user to bypass the IP address auto-discovery. This is necessary if the component resides on a computer
    # other than the computing running the backplane.

    parser = argparse.ArgumentParser()

    parser.add_argument("-a", dest="tcp_ip_address", default="None",
                        help="IP address TCP Server")
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane", )
    parser.add_argument("-e", dest="banyan_pub_topic", default="from_pico",
                        help="Topic for messages to the host PC")
    parser.add_argument("-g", dest="subscription_list", nargs='+', default="figura",
                        help="Banyan topics space delimited: topic1 topic2 topic3"),
    parser.add_argument("-l", dest="event_loop", default="None",
                        help="asyncio event loop")
    parser.add_argument("-n", dest="tcp_port", default=31335,
                        help="TCP Server Port Number")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")
    parser.add_argument("-z", dest="process_name", default="TcpGateway",
                        help="Name of this gateway")

    args = parser.parse_args()
    kw_options = {}

    if args.back_plane_ip_address != 'None':
        kw_options['back_plane_ip_address'] = args.back_plane_ip_address
    else:
        args.back_plane_ip_address = None

    if args.event_loop == 'None':
        args.event_loop = None
    else:
        kw_options['event_loop'] = args.event_loop

    kw_options = {
                  'back_plane_ip_address': args.back_plane_ip_address,
                  'tcp_server_ip_address': args.tcp_ip_address,
                  'tcp_server_ip_port': args.tcp_port,
                  'publisher_port': args.publisher_port,
                  'subscriber_port': args.subscriber_port,
                  'process_name': args.process_name,
                  'subscriber_list': args.subscription_list,
                  'banyan_pub_topic': args.banyan_pub_topic,
                  'event_loop': args.event_loop
                  }

    TcpGateWay(**kw_options)


# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    sys.exit(0)


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    tcp_gateway()



#!/usr/bin/env python3

"""
gateway_base_aio.py

 Copyright (c) 2017-2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import asyncio
import sys
from python_banyan.banyan_base_aio import BanyanBaseAIO


class GatewayBaseAIO(BanyanBaseAIO):
    """
    This class provides a common front end abstraction for all asyncio hardware gateways.
    """

    # pin modes
    DIGITAL_INPUT_MODE = 0
    DIGITAL_OUTPUT_MODE = 1
    PWM_OUTPUT_MODE = 2
    ANALOG_INPUT_MODE = 3
    ANALOG_OUTPUT_MODE = 4
    DIGITAL_INPUT_PULLUP_MODE = 5
    I2C_MODE = 6
    TONE_MODE = 7
    SERVO_MODE = 8
    STEPPER_MODE = 9
    SONAR_MODE = 10
    WILD_CARD_MODE = 11

    # index into pin database record
    PIN_MODE = 0
    LAST_VALUE = 1
    PULL_UP = 2

    # noinspection PyDefaultArgument,PyRedundantParentheses
    def __init__(self, back_plane_ip_address=None, subscriber_port='43125',
                 publisher_port='43124', process_name='',
                 subscriber_list=None, board_type=None, event_loop=None,):
        """

        :param back_plane_ip_address: banyan_base back_planeIP Address -
               if not specified, it will be set to the local computer

        :param subscriber_port: banyan_base back plane subscriber port.
               This must match that of the banyan_base backplane

        :param publisher_port: banyan_base back plane publisher port.
                               This must match that of the banyan_base
                               backplane

        :param process_name: Component identifier

        :param subscriber_list: a tuple or list of topics to be subscribed to

        :param board_type: micro-controller type ID

        :param event_loop: event loop that user may specify

        """
        if board_type:
            self.board_type = board_type

        if event_loop:
            self.event_loop = event_loop

        # fix for "not implemented" bugs in Python 3.8
        else:
            if sys.platform == 'win32':
                asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
            self.event_loop = asyncio.get_event_loop()

        if subscriber_list:
            self.subscriber_list = subscriber_list
        else:
            self.subscriber_list = ('all')

        # dictionaries for pin modes set by user
        # an entry consists of a pin number key
        # with a value of a list containing 3 values:
        #   1. the current pin mode
        #   2. the last reported value for this pin (for input modes only)
        #   3. a flag to state if pull up was enabled (for digital input only)

        # this dictionary initially contains an entry for each default
        # digital input pin

        self.pins_dictionary = {}

        # a pin can optionally be give a tag, it is used as a key to find pin number
        # tag(string): pin(integer)
        self.tags_dictionary = {}

        self.init_pins_dictionary()

        # for arduinos - save the pin number of the first analog pin
        self.first_analog_pin = 0

        # subscribe to initial topics
        # for topic in subscription_topics:
        #     await self.set_subscriber_topic(topic)
        # initialize the parent
        super(GatewayBaseAIO, self).__init__(back_plane_ip_address=back_plane_ip_address,
                                             subscriber_port=subscriber_port,
                                             publisher_port=publisher_port,
                                             process_name=process_name,
                                             subscriber_list=self.subscriber_list,
                                             event_loop=self.event_loop)

        self.command_dictionary = {'analog_write': self.analog_write,
                                   'digital_write': self.digital_write,
                                   'disable_analog_reporting': self.disable_analog_reporting,
                                   'disable_digital_reporting': self.disable_digital_reporting,
                                   'enable_analog_reporting': self.disable_analog_reporting,
                                   'enable_digital_reporting': self.disable_digital_reporting,
                                   'i2c_read': self.i2c_read,
                                   'i2c_write': self.i2c_write,
                                   'play_tone': self.play_tone,
                                   'pwm_write': self.pwm_write,
                                   'servo_position': self.servo_position,
                                   'set_mode_analog_input': self.set_mode_analog_input,
                                   'set_mode_digital_input': self.set_mode_digital_input,
                                   'set_mode_digital_input_pullup': self.set_mode_digital_input_pullup,
                                   'set_mode_digital_output': self.set_mode_digital_output,
                                   'set_mode_i2c': self.set_mode_i2c,
                                   'set_mode_pwm': self.set_mode_pwm,
                                   'set_mode_servo': self.set_mode_servo,
                                   'set_mode_sonar': self.set_mode_sonar,
                                   'set_mode_stepper': self.set_mode_stepper,
                                   'set_mode_tone': self.set_mode_tone,
                                   'stepper_write': self.stepper_write,
                                   }

    def init_pins_dictionary(self):
        """
        This method will initialize the pins dictionary
        This is handled within the class for each hardware type
        """
        raise NotImplementedError

    async def incoming_message_processing(self, topic, payload):
        """
        Messages are sent here from the receive_loop

        :param topic: Message Topic string

        :param payload: Message Data

        """
        # process payload command
        try:
            command = payload['command']
        except KeyError:
            print('incoming_message_processed KeyErrer', payload)
            raise

        # if a tag is provided and the tag is in the dictionary, fetch
        # the associated pin number
        if 'tag' in payload:
            tag = payload['tag']
            if tag:
                if tag in self.tags_dictionary:
                    pin = self.tags_dictionary[tag]
                    # the pin is optional if using tag, so add it to the payload
                    payload['pin'] = pin
                else:
                    self.tags_dictionary[payload['tag']] = payload['pin']

        # if command is in the command dictionary, execute the command
        if command in self.command_dictionary.keys():
            await self.command_dictionary[command](topic, payload)

        # for unknown requests, pass them along to the hardware gateway to handle
        else:
            await self.additional_banyan_messages(topic, payload)

    # all of the following methods should be overridden in the hardware
    # specific gateway when being used.

    async def additional_banyan_messages(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def analog_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def digital_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def disable_analog_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def disable_digital_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def enable_analog_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def enable_digital_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def i2c_read(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def i2c_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def play_tone(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def pwm_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def servo_position(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_analog_input(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_digital_input(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_digital_input_pullup(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_digital_output(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_i2c(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_pwm(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_servo(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_sonar(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_stepper(self, topic, payload):
        """
        Set the mode for the specific board.
        Must be overwritten by the hardware gateway class.

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def set_mode_tone(self, topic, payload):
        """
        Must be overwritten by the hardware gateway class.
        Handles a digital write

        :param topic: message topic

        :param payload: message payload
        """
        raise NotImplementedError

    async def digital_read(self, pin):
        """
        Read the state of a digital input pin.

        :param pin: pin number
        """
        raise NotImplementedError

    async def stepper_write(self, topic, payload):
        """
        Must be overwritten by the hardware gateway class.
        Handles a pwm write

        :param topic: message topic

        :param payload: message payload

        """
        raise NotImplementedError



#!/usr/bin/env python3

"""
gateway_base.py

 Copyright (c) 2017-2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

from python_banyan.banyan_base import BanyanBase


class GatewayBase(BanyanBase):
    """
    This class provides a common front end abstraction for all asyncio hardware gateways.
    """

    # pin modes
    DIGITAL_INPUT_MODE = 0
    DIGITAL_OUTPUT_MODE = 1
    PWM_OUTPUT_MODE = 2
    ANALOG_INPUT_MODE = 3
    ANALOG_OUTPUT_MODE = 4
    DIGITAL_INPUT_PULLUP_MODE = 5
    I2C_MODE = 6
    TONE_MODE = 7
    SERVO_MODE = 8
    STEPPER_MODE = 9
    SONAR_MODE = 10
    WILD_CARD_MODE = 11

    # target gpio device types types
    ARDUINO = 0
    RPi = 1
    ESP8266 = 2
    CRICKIT = 3

    # noinspection PyDefaultArgument,PyRedundantParentheses
    def __init__(self, back_plane_ip_address=None, subscriber_port='43125',
                 publisher_port='43124', process_name='',
                 subscriber_list=None, board_type=None, ):
        """

        :param back_plane_ip_address: banyan_base back_planeIP Address -
               if not specified, it will be set to the local computer

        :param subscriber_port: banyan_base back plane subscriber port.
               This must match that of the banyan_base backplane

        :param publisher_port: banyan_base back plane publisher port.
                               This must match that of the banyan_base
                               backplane

        :param process_name: Component identifier

        :param subscriber_list: a tuple or list of topics to be subscribed to

        :param board_type: micro-controller type ID

        """
        if board_type:
            self.board_type = board_type

        if subscriber_list:
            self.subscriber_list = subscriber_list
        else:
            self.subscriber_list = ('all')

        # dictionaries for pin modes set by user
        # an entry is board type specific

        # this dictionary initially contains an entry for each default
        # digital input pin

        self.pins_dictionary = {}

        # a pin can optionally be given a tag, it is used as a key to find
        # pin number
        # tag(string): pin(integer)
        self.tags_dictionary = {}

        self.init_pins_dictionary()

        # initialize the parent
        super(GatewayBase, self).__init__(back_plane_ip_address=back_plane_ip_address,
                                          subscriber_port=subscriber_port,
                                          publisher_port=publisher_port,
                                          process_name=process_name,
                                          )

        self.command_dictionary = {'analog_write': self.analog_write,
                                   'digital_write': self.digital_write,
                                   'disable_analog_reporting': self.disable_analog_reporting,
                                   'disable_digital_reporting': self.disable_digital_reporting,
                                   'enable_analog_reporting': self.disable_analog_reporting,
                                   'enable_digital_reporting': self.disable_digital_reporting,
                                   'i2c_read': self.i2c_read,
                                   'i2c_write': self.i2c_write,
                                   'play_tone': self.play_tone,
                                   'pwm_write': self.pwm_write,
                                   'servo_position': self.servo_position,
                                   'set_mode_analog_input': self.set_mode_analog_input,
                                   'set_mode_digital_input': self.set_mode_digital_input,
                                   'set_mode_digital_input_pullup': self.set_mode_digital_input_pullup,
                                   'set_mode_digital_output': self.set_mode_digital_output,
                                   'set_mode_i2c': self.set_mode_i2c,
                                   'set_mode_pwm': self.set_mode_pwm,
                                   'set_mode_servo': self.set_mode_servo,
                                   'set_mode_sonar': self.set_mode_sonar,
                                   'set_mode_stepper': self.set_mode_stepper,
                                   'set_mode_tone': self.set_mode_tone,
                                   'stepper_write': self.stepper_write,
                                   }

        if subscriber_list is not None:
            for topic in subscriber_list:
                self.set_subscriber_topic(topic)

    def init_pins_dictionary(self):
        """
        This method will initialize the pins dictionary
        This is handled within the class for each hardware type
        """
        raise NotImplementedError

    def incoming_message_processing(self, topic, payload):
        """
        Messages are sent here from the receive_loop
        :param topic: Message Topic string
        :param payload: Message Data
        :return:
        """
        # process payload command
        try:
            command = payload['command']
        except KeyError:
            print(payload)
            raise

        # if a tag is provided and the tag is in the dictionary, fetch
        # the associated pin number
        if 'tag' in payload:
            tag = payload['tag']
            if tag:
                if tag in self.tags_dictionary:
                    pin = self.tags_dictionary[tag]
                    # the pin is optional if using tag, so add it to the payload
                    payload['pin'] = pin
                else:
                    self.tags_dictionary[payload['tag']] = payload['pin']

        # if command is in the command dictionary, execute the command
        if command in self.command_dictionary.keys():
            self.command_dictionary[command](topic, payload)

        # for unknown requests, pass them along to the hardware gateway to handle
        else:
            self.additional_banyan_messages(topic, payload)

    # all of the following methods should be overridden in the hardware
    # specific gateway when being used.

    def additional_banyan_messages(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def analog_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def digital_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def disable_analog_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def disable_digital_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def enable_analog_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def enable_digital_reporting(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def i2c_read(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def i2c_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def play_tone(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def pwm_write(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def servo_position(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_analog_input(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_digital_input(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_digital_input_pullup(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_digital_output(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_i2c(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_pwm(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_servo(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_sonar(self, topic, payload):
        """
        This method will pass any messages not handled by this class to the
        specific gateway class. Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_stepper(self, topic, payload):
        """
        Set the mode for the specific board.
        Must be overwritten by the hardware gateway class.
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def set_mode_tone(self, topic, payload):
        """
        Must be overwritten by the hardware gateway class.
        Handles a digital write
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError

    def digital_read(self, pin):
        raise NotImplementedError

    def stepper_write(self, topic, payload):
        """
        Must be overwritten by the hardware gateway class.
        Handles a pwm write
        :param topic: message topic
        :param payload: message payload
        """
        raise NotImplementedError



"""
backplane.py


 Copyright (c) 2016-2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
import signal
import socket
import sys
import time
import argparse
import zmq
import zmq.utils.win32


# noinspection PyMethodMayBeStatic,PyBroadException
class BackPlane:
    """
    This class instantiates a ZeroMQ forwarder that acts as the python_banyan software backplane.
    All other components use a common TCP address to connect to the backplane and have their messages forwarded.

    See http://learning-0mq-with-pyzmq.readthedocs.io/en/latest/pyzmq/devices/forwarder.html for info on forwarder
    """

    def __init__(self, subscriber_port='43125', publisher_port='43124', backplane_name='',
                 loop_time=.001):
        """
        This is the initializer for the Python Banyan BackPlane class. The class must be instantiated
        before starting any other Python Banyan components

        :param subscriber_port: subscriber IP port number

        :param publisher_port: publisher IP port number

        :param backplane_name: name to appear on the console for this backplane

        :param loop_time: event loop idle timer
        """

        # get ip address of this machine

        # create a socket
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        # use the google dns to figure out the machine's address and use that address for the backplane.
        # this precludes the necessity of having a network configuration file.
        # noinspection PyPep8
        try:
            s.connect(('8.8.8.8', 1))
            self.bp_ip_address = s.getsockname()[0]
        except:
            self.bp_ip_address = '127.0.0.1'
        finally:
            s.close()

        print('\n******************************************')
        if backplane_name == "":
            print('Backplane IP address: ' + self.bp_ip_address)
        else:
            print(backplane_name + ' Backplane IP address: ' + self.bp_ip_address)
        print('Subscriber Port = ' + subscriber_port)
        print('Publisher  Port = ' + publisher_port)
        print('Loop Time = ' + str(loop_time) + ' seconds')
        print('******************************************')

        self.loop_time = loop_time
        # create a zmq instance for the backplane
        self.bp = zmq.Context()

        # establish bp as python_banyan ZMQ FORWARDER Device

        # setup up socket that all components will connect and publish to
        self.publish_to_bp = self.bp.socket(zmq.SUB)
        bind_string = 'tcp://' + self.bp_ip_address + ':' + publisher_port
        self.publish_to_bp.bind(bind_string)

        # Don't filter any incoming messages, just pass them through
        try:
            # for python 3
            self.publish_to_bp.setsockopt_string(zmq.SUBSCRIBE, '')
        except TypeError:
            # for python 2
            self.publish_to_bp.setsockopt(zmq.SUBSCRIBE, '')

        # setup socket that all subscribers will connect to
        self.subscribe_to_bp = self.bp.socket(zmq.PUB)
        bind_string = 'tcp://' + self.bp_ip_address + ':' + subscriber_port
        self.subscribe_to_bp.bind(bind_string)

        # instantiate the forwarder device
        try:
            with zmq.utils.win32.allow_interrupt(self.clean_up):
                zmq.device(zmq.FORWARDER, self.publish_to_bp, self.subscribe_to_bp)
        except KeyboardInterrupt:
            self.clean_up()
            sys.exit()

    def run_back_plane(self):
        """
        This method runs the backplane in a do nothing forever loop to keep the back plane alive.
        :return:
        """
        while True:
            try:
                time.sleep(self.loop_time)
            except KeyboardInterrupt:
                sys.exit(0)

    def clean_up(self):
        """
        Close the zmq publish and subscribe sockets and release the zmq context
        :return:
        """
        self.publish_to_bp.close()
        self.subscribe_to_bp.close()
        self.bp.term()


def bp():
    """
    Instantiate the backplane and run it.
    Attach a signal handler for the process to listen for user pressing Control C

    usage: backplane [-h] [-n BACKPLANE_NAME] [-p PUBLISHER_PORT] [-s SUBSCRIBER_PORT] [-t LOOP_TIME]

    optional arguments:

      -h, --help          show this help message and exit

      -n BACKPLANE_NAME   Name of this backplane

      -p PUBLISHER_PORT   Publisher IP port

      -s SUBSCRIBER_PORT  Subscriber IP port

      -t LOOP_TIME        Event Loop Timer in seconds

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("-n", dest="backplane_name", default="", help="Name of this backplane")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")
    parser.add_argument("-t", dest="loop_time", default=".001", help="Event Loop Timer in seconds")

    args = parser.parse_args()
    kw_options = {'publisher_port': args.publisher_port, 'subscriber_port': args.subscriber_port,
                  'backplane_name': args.backplane_name, 'loop_time': float(args.loop_time)}
    # replace with the name of your class
    backplane = BackPlane(**kw_options)
    backplane.run_back_plane()


# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)


if __name__ == '__main__':
    bp()



"""
banyan_base_multi.py

 Copyright (c) 2016 - 2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
from __future__ import unicode_literals

# Use argparse and signal if you wish to implement the argparse
# code located at the bottom of this file
# import argparse
# import signal

import csv
import sys
import time
import itertools
import msgpack
import msgpack_numpy as m
import zmq
import os



# noinspection PyMethodMayBeStatic
class BanyanBaseMulti(object):
    """

    This is the base class for Python Banyan components that wish to connect to multiple back planes.
    It requires the user to create a .csv descriptor file to describe the back planes and their
    addresses/ports and subscription topics.

    The .csv file has the following format. This line must be the first line in the file:

    backplane_name,ip_address,subscriber_port,subscriber_topic,publisher_port

    backplane_name: a unique identifier string for backplane - for informational purposes only

    ip_address: IP address of the computer that the backplane is running on

    subscriber_port: port number assigned to the backplane's subscriber port

    subscriber_topic: this is an optional list of subscription topics

    publisher_port: port number assigned to the backplane's publisher port



    There should be an entry in this file for each backplane that the component needs to connect to.

    This class encapsulates zeromq and message pack functionality. Its methods should be overridden by the user
    in the derived class to meet the needs of the component.

    To import into  the derived class use:

           from python_banyan.banyan_base_multi import BanyanBaseMulti

    """

    def __init__(self, back_plane_csv_file=None, process_name='None',
                 loop_time=.1, numpy=False, connect_time=0.3):
        """
        The __init__ method sets up all the ZeroMQ "plumbing"

        :param back_plane_csv_file: full path to .csv file with backplane descriptors

        :param process_name: identifier for your component printed at startup on the console

        :param loop_time: receive loop sleep time

        :param numpy: Set true if you wish to include numpy matrices in your messages

        :param connect_time: a short delay to allow the component to connect to the Backplane

        :return:
        """

        # socket type - used for calls to find_socket
        self.SUB_SOCK = 0
        self.PUB_SOCK = 1

        if back_plane_csv_file is None:
            raise ValueError('You must specify a valid .csv backplane descriptor file')

        # file specified, make sure it exists
        if not os.path.isfile(back_plane_csv_file):
            raise ValueError("Can't find backplane configuration file")

        if process_name == 'None':
            print('Warning: No Process Name Was Specified')

        self.numpy = numpy

        self.connect_time = connect_time

        # if using numpy apply the msgpack_numpy monkey patch
        if numpy:
            m.patch()

        self.loop_time = loop_time

        # get a zeromq context
        self.my_context = zmq.Context()

        # a list of dictionaries describing connections to the back planes
        self.backplane_table = []

        print("\nUsing Backplane Descriptor File: ", back_plane_csv_file)

        with open(back_plane_csv_file) as csvfile:
            reader = csv.DictReader(csvfile)
            print('\n************************************************************\n')
            for row in reader:
                # make sure backplane name is unique
                if any(d['backplane_name'] == row['backplane_name'] for d in self.backplane_table):
                    raise RuntimeError('Duplicate Back Plane Name - check your .csv file')

                print(process_name + ' using ' + row['backplane_name'] + ' Black plane at IP Address: ' +
                      row['ip_address'])

                # setup a publisher and subscriber for each backplane
                subscriber = None
                if row['subscriber_port']:
                    subscriber = self.my_context.socket(zmq.SUB)
                    connect_string = "tcp://" + row['ip_address'] + ':' + row['subscriber_port']
                    subscriber.connect(connect_string)

                publisher = None
                if row['publisher_port']:
                    publisher = self.my_context.socket(zmq.PUB)
                    connect_string = "tcp://" + row['ip_address'] + ':' + row['publisher_port']
                    publisher.connect(connect_string)

                # get topics and subscribe to them
                # test that topic string has a leading and trailing []
                if row['subscriber_port']:
                    print('    Subscriber Port = ' + row['subscriber_port'])
                    topic_list = row['subscriber_topic']
                    if '[' not in topic_list:
                        raise RuntimeError('Topic field must begin with "[" and end with "]" ')
                    if ']' not in topic_list:
                        raise RuntimeError('Topic field must begin with "[" and end with "]" ')

                    # make sure that the topic string does not contain a space character
                    if ' ' in topic_list:
                        raise RuntimeError('Topics may not contain a space character')

                    topic_list = topic_list[1:-1].split(',')

                    # subscribe to topics in list
                    for t in topic_list:
                        if sys.version_info[0] < 3:
                            t = t.encode()
                        print('        Subscribed to topic: ' + t)
                        self.set_subscriber_topic(t, subscriber)
                else:
                    print('    Subscriber Port = None Specified')

                if row['publisher_port']:
                    print('    Publisher  Port = ' + row['publisher_port'])
                else:
                    print('    Publisher  Port = None Specified')

                # update backplane table with new entry
                self.backplane_table.append(
                    {'backplane_name': row['backplane_name'], 'subscriber': subscriber,
                     'publisher': publisher})

            # wait for the last Backplane TCP connection
            time.sleep(self.connect_time)

            print()
            print('Loop Time = ' + str(loop_time) + ' seconds\n')
            print('************************************************************')

    def find_socket(self, backplane, socket_type):
        """
        Find a publisher or subscriber in the backplane table and return a ZMQ socket reference

        :param backplane: backplane name entry in table

        :param socket_type: publisher or subscriber

        :return: socket reference or None
        """
        valid_socket_types = [self.PUB_SOCK, self.SUB_SOCK]

        if socket_type in valid_socket_types:
            try:
                entry = next(item for item in self.backplane_table if item.get("backplane_name") == backplane)
                if socket_type == self.PUB_SOCK:
                    rval = entry['publisher']
                else:
                    rval = entry['subscriber']
                return rval

            except StopIteration:
                raise StopIteration(backplane + ' not found in table.')
        else:
            raise ValueError(socket_type + ' is an illegal socket_type')

    def set_subscriber_topic(self, topic, subscriber_socket):
        """
        This method sets a subscriber topic.

        You can subscribe to multiple topics by calling this method for
        each topic.

        :param topic: A topic string

        :param subscriber_socket: subscriber socket

        :return:
        """

        # make sure topic is a string
        if not type(topic) is str:
            raise TypeError('Subscriber topic must be python_banyan string')

        # does the subscriber socket exist?
        if subscriber_socket:
            subscriber_socket.setsockopt(zmq.SUBSCRIBE, topic.encode())
        else:
            raise ValueError('set_subscriber_topic: socket is None')

    def unsubscribe_topic(self, topic, subscriber_socket):
        """
        This method un-subscribes from a topic.

        :param topic: A topic string

        :param subscriber_socket: subscriber socket

        :return:
        """

        # make sure topic is a string
        if not type(topic) is str:
            raise TypeError('Subscriber topic must be python_banyan string')

        # make sure that a socket reference has been passed in
        if subscriber_socket:
            subscriber_socket.unsubscribe(topic.encode())
        else:
            raise ValueError('set_subscriber_topic: socket is None')

    def publish_payload(self, payload, publisher_socket, topic=''):
        """
        This method will publish a python_banyan payload and its associated topic

        :param payload:  Protocol message to be published

        :param publisher_socket: Publisher socket - handle to socket or "BROADCAST" to send to
                                 all connected publisher sockets

        :param topic: A string value for message topic

        :return:
        """

        # make sure topic is a string
        if not type(topic) is str:
            raise TypeError('Publish topic must be python_banyan string', 'topic')

        # create python_banyan message pack payload
        if self.numpy:
            message = msgpack.packb(payload, default=m.encode)
        else:
            message = msgpack.packb(payload)

        pub_envelope = topic.encode()
        if publisher_socket == "BROADCAST":
            for element in self.backplane_table:
                if element['publisher']:
                    element['publisher'].send_multipart([pub_envelope, message])
        else:

            if publisher_socket:
                publisher_socket.send_multipart([pub_envelope, message])
            else:
                raise ValueError('Invalid publisher socket')

    def receive_loop(self):
        """
        This is the receive loop for zmq messages.

        This method may be overwritten to meet the needs
        of the application before handling received messages.

        :return:
        """
        for element in itertools.cycle(self.backplane_table):
            if element['subscriber']:
                try:
                    data = element['subscriber'].recv_multipart(zmq.NOBLOCK)
                    if self.numpy:
                        payload = msgpack.unpackb(data[1], object_hook=m.decode)
                        self.incoming_message_processing(data[0].decode(), payload)
                    else:
                        self.incoming_message_processing(data[0].decode(), umsgpack.unpackb(data[1]))
                except zmq.error.Again:
                    try:
                        time.sleep(self.loop_time)
                    except KeyboardInterrupt:
                        self.clean_up()
                        sys.exit(0)
                except AttributeError:
                    raise

    def incoming_message_processing(self, topic, payload):
        """
        Override this method with a custom python_banyan message processor for subscribed messages

        :param topic: Message Topic string

        :param payload: Message Data

        :return:
        """
        print('this method should be overwritten in the child class', topic, payload)

    def clean_up(self):
        """
        Clean up before exiting - override if additional cleanup is necessary

        :return:
        """
        for element in self.backplane_table:
            if element['publisher']:
                element['publisher'].close()
            if element['subscriber']:
                element['subscriber'].close()
        self.my_context.term()

# When creating a derived component, replicate the code below and replace banyan_base_multi with a  name of your choice

# def banyan_base_multi():
#     # allow user to bypass the IP address auto-discovery. This is necessary if the component resides on a computer
#     # other than the computing running the backplane.
#
#     parser = argparse.ArgumentParser()
#     parser.add_argument("-b", dest="back_plane_csv_file", default="None",
#                         help="Backplane .csv descriptor file name")
#
#     # allow the user to specify a name for the component and have it shown on the console banner.
#     # modify the default process name to one you wish to see on the banner.
#     # change the default in the derived class to set the name
#     parser.add_argument("-n", dest="process_name", default="YOUR PROCESS NAME", help="Set process name in banner")
#
#     parser.add_argument("-t", dest="loop_time", default=".1", help="Event Loop Timer in seconds")
#
#     args = parser.parse_args()
#     kw_options = {}
#
#     if args.back_plane_csv_file != 'None':
#         kw_options['back_plane_csv_file'] = args.back_plane_csv_file
#
#     kw_options['process_name'] = args.process_name
#
#     kw_options['loop_time'] = float(args.loop_time)
#
#     # replace with the name of your class
#     app = BanyanBaseMulti(**kw_options)
#
#     # optionally start the receive loop here or start it in your __init__
#     app.receive_loop()
#
#
# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
# def signal_handler(sig, frame):
#     print('Exiting Through Signal Handler')
#     raise KeyboardInterrupt
#
#
# # listen for SIGINT
# signal.signal(signal.SIGINT, signal_handler)
# signal.signal(signal.SIGTERM, signal_handler)
#
#
# if __name__ == '__main__':
#     # replace with name of function you defined above
#     banyan_base_multi()



#!/usr/bin/env python3

"""
monitor.py

Copyright (c) 2016 - 2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

# monitor.py

import argparse
import signal
import sys

import zmq

from python_banyan.banyan_base import BanyanBase


# noinspection PyMethodMayBeStatic
class Monitor(BanyanBase):
    """
    This class subscribes to all messages on the back plane and prints out both topic and payload.
    """

    def __init__(self, back_plane_ip_address=None,
                 subscriber_port='43125', publisher_port='43124', process_name=None,
                 numpy=False):
        """
        This is constructor for the Monitor class

        :param back_plane_ip_address: IP address of the currently running backplane

        :param subscriber_port: subscriber port number - matches that of backplane

        :param publisher_port: publisher port number - matches that of backplane

        :param process_name: default name is "Monitor". Change using this parameter
        """

        # initialize the base class
        super(Monitor, self).__init__(back_plane_ip_address, subscriber_port,
                                      publisher_port, process_name=process_name,
                                      # loop_time=0.0,
                                      numpy=numpy)

        self.set_subscriber_topic('')

        # receive loop is defined in the base class
        try:
            self.receive_loop()
        except zmq.error.ZMQError:
            sys.exit()
        except KeyboardInterrupt:
            sys.exit()

    def incoming_message_processing(self, topic, payload):
        """
        This method is overwritten in the inherited class to process the data_files
        :param topic: Message topic string
        :param payload: Message content
        :return:
        """
        print(topic, payload)


def monitor():
    # noinspection PyShadowingNames

    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane")
    parser.add_argument("-m", dest="numpy", default="False", help="Set to True for numpy matrices")
    parser.add_argument("-n", dest="process_name", default="Monitor", help="Set process name in banner")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")

    args = parser.parse_args()
    kw_options = {}

    if args.back_plane_ip_address != 'None':
        kw_options['back_plane_ip_address'] = args.back_plane_ip_address

    kw_options['process_name'] = args.process_name

    kw_options['publisher_port'] = args.publisher_port
    kw_options['subscriber_port'] = args.subscriber_port

    if args.numpy == "True":
        kw_options['numpy'] = True
    else:
        kw_options['numpy'] = False

    Monitor(**kw_options)

# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt

# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)


if __name__ == '__main__':
    monitor()



#!/usr/bin/env python3

"""
 Copyright (c) 2018-2019 Alan Yorinks - All Rights Reserved.


 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import argparse
import copy
import csv
import logging
import signal
import subprocess
import sys
import time
from os.path import expanduser
from subprocess import Popen

import psutil
from apscheduler.schedulers.background import BackgroundScheduler

from python_banyan.banyan_base import BanyanBase


# noinspection PyTypeChecker
class BLS(BanyanBase):
    """
    This is the banyan launcher server. It reads a user .csv
    processor descriptor file. This file describes all processes
    to be launched locally as well as remotely.

    Format for the file is described as:
    command_string,spawn,topic,append_bp_address,auto_restart

    Where:
        command string is the command to issue
        spawn = yes or no. If yes the command is spawned in its own window
        topic = local if the process should be run on this computer.
                For clients, it is the topic they will be listening for
                to get their launch instructions
        append_bp_address = yes or no. If yes a -b command line option with
                            the IP address of this computer is appended to the
                            command string.
        auto_restart = yes or no. If yes, if the process dies it will be restarted.

    """

    def __init__(self, subscriber_port='43125', publisher_port='43124',
                 descriptor_file='launch.csv', ):
        """
        Initialize the launch server
        :param subscriber_port: backplane subscriber port
        :param publisher_port:  backplane publisher port
        :param descriptor_file: .csv process descriptor file
        """

        # get the path to the home directory
        home = expanduser("~")

        # start the logging process
        # the log file will be found in the user's home directory

        logging.basicConfig(filename=home + '/banyan_launcher.log', filemode='w',
                            level=logging.ERROR)

        # a popen process object
        self.proc = None

        # create a launch_id counter
        self.launch_id = 1

        # maintain a database of information pertaining to each launch item
        # this will be an array of dictionaries, with each row describing a single launch process
        self.launch_db = []

        # the keys defined for a given row are as follows:
        #
        # auto_restart - restart process if it dies
        # append_bp_address - the backplane ip address appended to
        #                     the command_string with -b option
        # command_string - the command used to launch the process
        # process - the value returned from popen after launching process
        # process_id - pid of launched process
        # spawn - spawn process in its own window
        # topic - used to publish to remote launcher
        # reply_topic - reply topic from remote launcher
        # launch_id - id so that remote can differentiate multiple instances of a component

        self.empty_launch_entry = {'append_bp_address': 'no', 'auto_restart': 'no',
                                   'command_string': None, 'process': None, 'process_id': None,
                                   'spawn': 'no', 'topic': None, 'reply_topic': None,
                                   'launch_id': None
                                   }

        # create an instance of an empty launch_db entry
        new_entry = copy.deepcopy(self.empty_launch_entry)

        # backplane status string
        bp_string = ""

        # checking running processes.
        # if the backplane is already running, just note that and move on.
        for pid in psutil.pids():
            p = psutil.Process(pid)
            if p.name() == "backplane":
                new_entry['process_id'] = p
                bp_string = "Backplane already running.          PID = " + str(p.pid)
                self.backplane_exists = True
            else:
                continue

        # if backplane is not already running, start a new instance
        if not new_entry['process_id']:
            self.proc = Popen(['backplane'],
                              stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                              stdout=subprocess.PIPE)
            new_entry['process'] = self.proc
            new_entry['process_id'] = self.proc.pid

        # prepare an entry for the launch_db table
        new_entry['command_string'] = 'backplane'
        new_entry['wait'] = '0'

        # add the backplane entry to the launch database
        self.launch_db.append(new_entry)

        # call the parent class to attach this banyan component to the backplane
        super(BLS, self).__init__(subscriber_port=subscriber_port,
                                  publisher_port=publisher_port, process_name='Banyan Launch Server',
                                  loop_time=.1)

        # read in the descriptor file
        with open(descriptor_file) as csvfile:
            reader = csv.DictReader(csvfile)
            # add a field for pid
            for row in reader:
                # get an empty entry for launch_db
                new_entry = copy.deepcopy(self.empty_launch_entry)
                # add all fields to an empty launch database entry
                # add additional fields not specified in the csv file
                new_entry['topic'] = row['topic']
                new_entry['auto_restart'] = row['auto_restart']
                new_entry['append_bp_address'] = row['append_bp_address']
                new_entry['command_string'] = row['command_string']
                new_entry['spawn'] = row['spawn']
                new_entry['wait'] = row['wait']

                # if the topic is not 'local' then create a reply topic
                # and check if the command needs to have a -b option appended
                if not row['topic'] == 'local':
                    new_entry['reply_topic'] = 'ACK' + new_entry['topic']
                    # print(new_entry['reply_topic'])
                    self.set_subscriber_topic(new_entry['reply_topic'])
                    if new_entry['append_bp_address'] == 'yes':
                        new_entry['command_string'] = new_entry['command_string'] + ' -b ' + self.back_plane_ip_address
                    new_entry['launch_id'] = self.launch_id
                    self.launch_id += 1

                # add the populated record to launch_db
                self.launch_db.append(new_entry)

        # display pid of backplane either just invoked or already running
        if not bp_string:
            print('backplane                           PID = {}'.format(str(self.proc.pid)))
        else:
            print(bp_string)

        # launch local processes or send launch messages to remotes
        for idx, record in enumerate(self.launch_db):
            # skip over the backplane entry
            if not record['command_string'] == 'backplane':
                #
                if record['topic'] == 'local':
                    self.spawn_local(idx)
                else:
                    self.publish_payload(record, record['topic'])
            try:
                if int(record['wait']) > 0:
                    time.sleep(int(record['wait']))
            except TypeError:
                print(record)
                raise

        # start the background scheduler to periodically run check_processes and confirm methods
        self.scheduler = BackgroundScheduler()
        self.job = self.scheduler.add_job(self.check_local_processes, 'interval', seconds=.5)
        self.job2 = self.scheduler.add_job(self.confirm_remote, 'interval', seconds=5)

        # subscribe to the killall topic to exit when received
        self.set_subscriber_topic('killall')

        # start the schedule running
        self.scheduler.start()

        try:
            # initial launching is complete, so just wait to receive incoming messages.
            self.receive_loop()
        except (KeyboardInterrupt, SystemExit):
            self.clean_up()

    def spawn_local(self, idx):
        """
        This method launches processes that are needed to run on this computer
        :param idx: An index into launch_db
        """

        # get the launch entry in launch_db
        db_entry = self.launch_db[idx]

        # skip over the entry for the backplane
        # launch the process either in its own window or just launch it.
        # differentiate between windows and other os's.
        if not db_entry['command_string'] == 'backplane':
            if sys.platform.startswith('win32'):
                if db_entry['spawn'] == 'yes':
                    self.proc = Popen(db_entry['command_string'],
                                      creationflags=subprocess.CREATE_NEW_CONSOLE)
                else:
                    command_list = db_entry['command_string']
                    self.proc = Popen(command_list, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
            else:
                if db_entry['spawn'] == 'yes':
                    self.proc = Popen(['xterm', '-e', db_entry['command_string']],
                                      stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                      stdout=subprocess.PIPE)
                else:
                    command_list = db_entry['command_string'].split(' ')
                    self.proc = Popen(command_list)

            # update the entry with the launch information
            db_entry['process'] = self.proc
            db_entry['process_id'] = self.proc.pid
            print('{:35} PID = {}'.format(db_entry['command_string'], str(self.proc.pid)))

            # allow a little time for the process to startup
            try:
                time.sleep(0.5)
            except (KeyboardInterrupt, SystemExit):
                self.clean_up()

    def check_local_processes(self):
        """
        This method is called by the scheduler periodically.
        Check to make sure if a local process previously started is still running.
        If a process is dead, print that to the console and if the process
        has the restart flag set, then restart it.

        Affect the launch_db entry so that only one message is printed and the if
        the process to be restarted, it is restarted once.
        """

        if sys.platform.startswith('win32'):
            for x, record in enumerate(self.launch_db):

                # ignore backplane
                if not record['command_string'] == 'backplane':
                    # get a list of all pids running
                    pids = psutil.pids()

                    # status = None
                    # check to see if the process in the launch_db is in this list
                    if record['process'].pid in pids:
                        proc = psutil.Process(record['process'].pid)

                        # get the status for this process
                        status = proc.status()

                    elif not record['process'].pid in pids:
                        status = psutil.STATUS_ZOMBIE

                    else:
                        status = None

                    # print(status)
                    if status == psutil.STATUS_ZOMBIE:
                        log_string = '{:35} PID = {} DIED'.format(record['command_string'], str(record['process'].pid))
                        print(log_string)

                        # log_string = record['command_string'] + " PID = " + str(record['process'].pid + 'DIED')
                        logger = logging.getLogger()
                        logger.error(log_string)

                        # reset its state and process, and process ID
                        record['process'] = None
                        record['process_id'] = None

                        # do we need to restart it?
                        if record['auto_restart'] == 'yes':
                            self.spawn_local(x)
        else:
            for x, record in enumerate(self.launch_db):
                if not record['command_string'] == 'backplane':
                    if record['process_id']:
                        pids = psutil.pids()
                        # print(record)
                        # check to see if the process in the launch_db is in this list
                        # status = None

                        try:
                            if record['process'].pid in pids:
                                proc = psutil.Process(record['process'].pid)
                                status = proc.status()
                                # print('command: ' + record['command_string'] + 'status: ' + status)

                                if status == psutil.STATUS_SLEEPING:
                                    continue

                                if not status:
                                    status = psutil.STATUS_ZOMBIE

                                if status == psutil.STATUS_ZOMBIE:

                                    log_string = '{:35} PID = {} DIED'.format(record['command_string'],
                                                                              str(record['process'].pid))
                                    print(log_string)

                                    logger = logging.getLogger()
                                    logger.error(log_string)

                                    # reset its process, and process ID
                                    record['process'] = None
                                    record['process_id'] = None

                                    # do we need to restart it?
                                    if record['auto_restart'] == 'yes':
                                        self.spawn_local(x)
                        except AttributeError:
                            pass

    def incoming_message_processing(self, topic, payload):
        """
        Messages are sent here from the receive_loop

        :param topic: Message Topic string
        :param payload: Message Data
        :return:
        """
        print(topic)
        if topic == 'killall':
            self.clean_up()

        # update the launch_db record for the remote process so that we do not send additional launch commands
        for idx, record in enumerate(self.launch_db):
            # if no launch id has already been assigned, update the record and print a message
            # if not record['launch_id']:
            # is this an acknowledgement from the remote launch?
            if record['launch_id']:
                if record['launch_id'] == payload['launch_id']:
                    self.launch_db[idx]['process_id'] = payload['process_id']
                    print('{}: {:25} PID = {}'.format(topic, record['command_string'], str(payload['process_id'])))
                    break

    def confirm_remote(self):
        """
        This method checks to see if the remote sides acknowledged starting their processes.
        If not, resend them
        :return:
        """

        for idx, record in enumerate(self.launch_db):
            if not record['process_id']:
                if record['topic'] != 'local':
                    self.publish_payload(record, record['topic'])

    def clean_up(self):
        """
        Graceful shutdown - all newly opened windows and associated processes
        are killed
        :return:
        """
        self.scheduler.pause()

        for idx, record in enumerate(self.launch_db):
            if record['process']:

                print('{:35} PID = {} KILLED'.format(record['command_string'], str(record['process'].pid)))
                try:
                    proc = psutil.Process(record['process'].pid)
                    for c in proc.children(recursive=True):
                        c.kill()
                    proc.kill()
                    record['process'] = None
                    record['process_id'] = None
                    # this is necessary for killing the command windows in Windows
                    if proc.name() == record['command_string']:
                        proc.kill()
                except psutil.NoSuchProcess:
                    pass

        super(BLS, self).clean_up()
        sys.exit(0)


def bls():
    # allow user to bypass the IP address auto-discovery. This is necessary if the component resides on a computer
    # other than the computing running the backplane.

    parser = argparse.ArgumentParser()

    parser.add_argument("-f", dest="descriptor_file", default='launch.csv',
                        help="csv descriptor file")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")

    args = parser.parse_args()

    kw_options = {
        'publisher_port': args.publisher_port,
        'subscriber_port': args.subscriber_port,
        'descriptor_file': args.descriptor_file, }

    # replace with the name of your class
    BLS(**kw_options)


# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    # replace with name of function you defined above
    bls()



"""
mqtt_gateway.py

 Copyright (c) 2018 - 2019 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import argparse
import signal
import sys
import zmq
import json

from python_banyan.banyan_base import BanyanBase
# noinspection PyPackageRequirements
import paho.mqtt.client as mqtt


# noinspection PyUnusedLocal
class MqttGateway(BanyanBase):
    """
    This class allows a subscription to multiple MQTT topics published on an MQTT
    broker and will republish these messages to the Banyan backplane.

    This class also allows a Banyan application to publish messages to an MQTT broker.
    The MQTT topic is set to the value of the mqtt_pub_topic parameter. The MQTT payload
    is the same as the Banyan payload
    """

    # noinspection PyUnusedLocal
    def __init__(self, back_plane_ip_address=None,
                 subscriber_port='43125', publisher_port='43124',
                 process_name='MQTT Gateway', loop_time=.0001,
                 numpy=False,
                 banyan_sub_topics='to_mqtt',
                 banyan_pub_topic='from_mqtt',
                 mqtt_addr='localhost',
                 mqtt_port=1883,
                 mqtt_sub_topics=None,
                 mqtt_pub_topic='from_banyan'):
        """

        :param back_plane_ip_address: Banyan backplane IP address
        :param subscriber_port: Banyan subscriber IP port
        :param publisher_port: Banyan publisher IP port
        :param process_name: Name to display in this component's header
        :param loop_time: Banyan idle loop time
        :param numpy: Numpy flag
        :param banyan_sub_topics: A list of Banyan topics we wish to receive
                                  that will forwarded to the MQTT network.
        :param banyan_pub_topic: Topic string for publishing "MQTT messages"
                                 to the Banyan network
        :param mqtt_addr: MQTT IP address
        :param mqtt_port: MQTT port
        :param mqtt_sub_topics: A list of MQTT topics we wish to receive from MQTT
                                and will forwarded to the Banyan network.
        :param mqtt_pub_topic: Topic string for publishing "Banyan messages"
                               to the MQTT network
        """

        # initialize the base class
        super(MqttGateway, self).__init__(back_plane_ip_address, subscriber_port,
                                          publisher_port, process_name='MqttGateway',
                                          loop_time=0.1,
                                          numpy=numpy)

        # save the publication topics
        self.banyan_pub_topic = banyan_pub_topic
        self.mqtt_pub_topic = mqtt_pub_topic

        # save the banyan subscription topics
        self.banyan_sub_topics = banyan_sub_topics

        # make sure it is in the form of a list
        if type(self.banyan_sub_topics) is not list:
            self.banyan_sub_topics = [self.banyan_sub_topics]

        # these topics are used by banyan components when they wish to publish a payload
        # to mqtt
        if self.banyan_sub_topics:
            for sub in self.banyan_sub_topics:
                self.set_subscriber_topic(sub)

        # save the mqtt topic list, ip addr and publication topic
        self.mqtt_sub_topics = mqtt_sub_topics

        # make sure it is in the form of a list
        if type(self.mqtt_sub_topics) is not list:
            self.mqtt_sub_topics = [self.mqtt_sub_topics]

        self.mqtt_addr = mqtt_addr
        self.mqtt_port = mqtt_port

        # establish the mqtt client and connect
        self.client = mqtt.Client()
        self.client.connect(self.mqtt_addr, self.mqtt_port, 60)
        self.client.on_connect = self.mqtt_on_connect
        self.client.on_message = self.mqtt_on_message
        self.client.loop_start()

        try:
            self.receive_loop()
        except zmq.error.ZMQError:
            sys.exit()
        except KeyboardInterrupt:
            sys.exit()

    def mqtt_on_connect(self, client, userdata, flags, rc):
        """
        standard mqtt on connect - see mqtt documentation for
        meaning of parameters
        :param client:
        :param userdata:
        :param flags:
        :param rc:
        :return:
        """
        print("MQTT Gateway Connected to MQTT {}:{} with result code {}.".format(self.mqtt_addr,
                                                                                 self.mqtt_port,
                                                                                 str(rc)))
        # when mqtt is connected to subscribe to mqtt topics
        if self.mqtt_sub_topics:
            for sub in self.mqtt_sub_topics:
                self.client.subscribe(sub)

    def mqtt_on_message(self, client, userdata, msg):
        """
        Republish mqtt messages in a Banyan format.
        See mqtt documentation for
        meaning of parameters.
        :param client:
        :param userdata:
        :param msg:
        :return:
        """
        m = msg.payload.decode()
        payload = json.loads(m)
        self.publish_payload(payload, self.banyan_pub_topic)

    def incoming_message_processing(self, topic, payload):
        """
        Publish a banyan message to the mqtt network
        :param topic: topic string
        :param payload: payload
        :return:
        """
        if topic == 'to_mqtt':
            payload = json.dumps(payload).encode()
            self.client.publish(self.mqtt_pub_topic, payload)


def mqtt_gateway():
    # allow user to bypass the IP address auto-discovery. This is necessary if the component resides on a computer
    # other than the computing running the backplane.

    parser = argparse.ArgumentParser()

    parser.add_argument("-a", dest="mqtt_ip_address", default="localhost",
                        help="IP address of mqtt broker")
    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane", )
    parser.add_argument("-d", dest="mqtt_port", default=1883,
                        help="MQTT Port Number")
    parser.add_argument("-e", dest="banyan_pub_topic", default="from_mqtt",
                        help="Topic for messages to MQTT")
    parser.add_argument("-g", dest="banyan_sub_topics", nargs='+', default="to_mqtt",
                        help="Banyan topics space delimited: topic1 topic2 topic3"),
    parser.add_argument("-i", dest="mqtt_pub_topic", default="from_banyan",
                        help="Topic for messages sent to MQTT")
    parser.add_argument("-j", dest="mqtt_sub_topics", nargs='+', default="mqtt_network",
                        help="MQTT topics space delimited: topic1 topic2 topic3")
    parser.add_argument("-n", dest="process_name", default="MQTT Gateway", help="Set process name in banner")
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")
    parser.add_argument("-t", dest="loop_time", default=".1", help="Event Loop Timer in seconds")

    args = parser.parse_args()
    kw_options = {}

    if args.back_plane_ip_address != 'None':
        kw_options['back_plane_ip_address'] = args.back_plane_ip_address

    kw_options = {'publisher_port': args.publisher_port,
                  'subscriber_port': args.subscriber_port,
                  'process_name': args.process_name,
                  'loop_time': float(args.loop_time),
                  'banyan_sub_topics': args.banyan_sub_topics,
                  'banyan_pub_topic': args.banyan_pub_topic,
                  'mqtt_addr': args.mqtt_ip_address,
                  'mqtt_port': args.mqtt_port,
                  'mqtt_sub_topics': args.mqtt_sub_topics,
                  'mqtt_pub_topic': args.mqtt_pub_topic,
                  }

    # replace with the name of your class
    MqttGateway(**kw_options)


# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    mqtt_gateway()



"""
 Copyright (c) 2020 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import asyncio
import sys


# noinspection PyStatementEffect,PyUnresolvedReferences,PyUnresolvedReferences
class TcpSocket:
    """
    This class encapsulates management of a tcp/ip connection that communicates
    with a TCP server
    """

    def __init__(self, ip_address, ip_port, loop):
        self.ip_address = ip_address
        self.ip_port = ip_port
        self.loop = loop
        self.reader = None
        self.writer = None

    async def start(self):
        """
        This method opens an IP connection on the IP device

        :return: None
        """
        connected = None
        print("Waiting to connect to the server ...")
        while not connected:
            try:
                self.reader, self.writer = await asyncio.open_connection(
                    self.ip_address, self.ip_port)
                print(f'Successfully connected to: {self.ip_address}:{self.ip_port}')
                connected = True
            except OSError:
                continue

    async def write(self, data):
        """
        This method writes sends data to the IP device
        :param data:

        :return: None
        """
        # we need to convert data formats,
        # so all of the below.
        output_list = []

        # create an array of integers from the data to be sent
        for x in data:
            # output_list.append((ord(x)))
            output_list.append(x)

        # now convert the integer list to a bytearray
        to_wifi = bytearray(output_list)
        self.writer.write(to_wifi)
        await self.writer.drain()

    async def read(self, num_bytes=1):
        """
        This method reads one byte of data from IP device

        :return: Next byte
        """
        buffer = await self.reader.read(num_bytes)
        return buffer



#!/usr/bin/env python3

"""
 Copyright (c) 2018-2019 Alan Yorinks - All Rights Reserved.


 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import argparse
import logging
import signal
import subprocess
import sys
import time
from os.path import expanduser
from subprocess import Popen

import psutil
from apscheduler.schedulers.background import BackgroundScheduler
from python_banyan.banyan_base import BanyanBase


class BLC(BanyanBase):
    """
    This is the banyan launcher client. It receives launch instructions
    from the server and then locally launches and manages those processes.
    """

    def __init__(self, subscriber_port='43125', publisher_port='43124',
                 back_plane_ip_address=None, topic=None):
        """
        :param back_plane_ip_address: address of backplane. This is a required
                                      parameter
        :param subscriber_port: backplane subscriber port
        :param publisher_port:  backplane publisher port
        :param topic: subscriber topic containing launch instructions. This is
                      a required parameter
        """

        # To use this class the backplane address is a required parameter
        if not any((back_plane_ip_address, topic)):
            print('You must specify both the backplane ip address and topic.')
            sys.exit(0)

        # start the logging process

        # get the home directory path
        home = expanduser("~")

        # set up the log file
        logging.basicConfig(filename=home + '/banyan_launcher.log', filemode='w',
                            level=logging.ERROR)

        # a popen process object
        self.proc = None

        # maintain a database of information pertaining to each launch item
        # this will be an array of dictionaries, with each row describing a single launched process
        self.launch_db = []

        # the keys defined for a given row are as follows:
        #
        # auto_restart - restart process if it dies
        # append_bp_address - the backplane ip address appended to
        #                     the command_string with -b option
        # command_string - the command used to launch the process
        # process - the value returned from popen after launching process
        # process_id - pid of launched process
        # spawn - spawn process in its own window
        # topic - used to publish to remote launcher
        # reply_topic - reply topic from remote launcher

        # call the parent class to attach this banyan component to the backplane
        super(BLC, self).__init__(back_plane_ip_address=back_plane_ip_address,
                                  subscriber_port=subscriber_port,
                                  publisher_port=publisher_port,
                                  process_name='Banyan Launch Client',
                                  loop_time=.1)

        print('Listening for ' + topic + ' messages.')

        # subscribe to the launch topic specified in the init parameter
        self.set_subscriber_topic(topic)

        # subscribe to the killall topic to exit this program via message
        self.set_subscriber_topic('killall')

        # start the background scheduler to periodically run check_processes and confirm
        self.scheduler = BackgroundScheduler()
        self.job = self.scheduler.add_job(self.check_local_processes, 'interval', seconds=.5)

        self.scheduler.start()

        try:
            # initial launching is complete, so just wait to receive incoming messages.
            self.receive_loop()
        except (KeyboardInterrupt, SystemExit):
            self.clean_up()

    def spawn_local(self, idx):
        """
        This method launches processes that are needed to run on this computer.
        :param idx: An index into launch_db
        """

        # get the launch entry in launch_db
        db_entry = self.launch_db[idx]

        # skip over the entry for the backplane.
        # there shouldn't be one for the client, but the code is
        # kept consist with the server.
        # launch the process either in its own window or just launch it.
        # differentiate between windows and other os's.
        if not db_entry['command_string'] == 'backplane':
            if sys.platform.startswith('win32'):
                if db_entry['spawn'] == 'yes':
                    self.proc = Popen(db_entry['command_string'],
                                      creationflags=subprocess.CREATE_NEW_CONSOLE)
                else:
                    command_list = db_entry['command_string']
                    self.proc = Popen(command_list, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
            else:
                if db_entry['spawn'] == 'yes':
                    self.proc = Popen(['xterm', '-e', db_entry['command_string']],
                                      stdin=subprocess.PIPE, stderr=subprocess.PIPE,
                                      stdout=subprocess.PIPE)
                else:
                    command_list = db_entry['command_string'].split(' ')
                    self.proc = Popen(command_list)

            # update the entry with the launch information
            db_entry['process'] = self.proc
            db_entry['process_id'] = self.proc.pid
            print('{:35} PID = {}'.format(db_entry['command_string'], str(self.proc.pid)))

            # allow a little time for the process to startup
            try:
                time.sleep(0.5)
            except (KeyboardInterrupt, SystemExit):
                # self.scheduler.shutdown()
                self.clean_up()
                sys.exit(0)
    """
    def check_local_processes(self):

        # This method is called by the scheduler periodically.
        # Check to make sure if a local process previously started is still running.
        # If a process is not dead, print that to the console and if the process
        # has the restart flag set, then restart it.
        
        for x, record in enumerate(self.launch_db):

            # ignore backplane
            if not record['command_string'] == 'backplane':
                # get a list of all pids running
                pids = psutil.pids()

                status = None
                # check to see if the process in the launch_db is in this list
                if record['process'].pid in pids:
                    proc = psutil.Process(record['process'].pid)

                    # get the status for this process
                    status = proc.status()

                # if it is not in the list, declare it a zombie to force the print
                if not record['process'].pid in pids:
                    status = psutil.STATUS_ZOMBIE

                if status == psutil.STATUS_ZOMBIE:
                    log_string = '{:35} PID = {} DIED'.format(record['command_string'], str(record['process'].pid))
                    print(log_string)

                    # log_string = record['command_string'] + " PID = " + str(record['process'].pid + 'DIED')
                    logger = logging.getLogger()
                    logger.error(log_string)

                    # reset its state and process, and process ID
                    record['process'] = None
                    record['process_id'] = None

                    # do we need to restart it?
                    if record['auto_restart'] == 'yes':
                        self.spawn_local(x)
    """
    def check_local_processes(self):
        """
        This method is called by the scheduler periodically.
        Check to make sure if a local process previously started is still running.
        If a process is dead, print that to the console and if the process
        has the restart flag set, then restart it.

        Affect the launch_db entry so that only one message is printed and the if
        the process to be restarted, it is restarted once.
        """

        if sys.platform.startswith('win32'):
            for x, record in enumerate(self.launch_db):

                # ignore backplane
                if not record['command_string'] == 'backplane':
                    # get a list of all pids running
                    pids = psutil.pids()

                    # status = None
                    # check to see if the process in the launch_db is in this list
                    if record['process'].pid in pids:
                        proc = psutil.Process(record['process'].pid)

                        # get the status for this process
                        status = proc.status()

                    elif not record['process'].pid in pids:
                        status = psutil.STATUS_ZOMBIE

                    else:
                        status = None

                    # print(status)
                    if status == psutil.STATUS_ZOMBIE:
                        log_string = '{:35} PID = {} DIED'.format(record['command_string'], str(record['process'].pid))
                        print(log_string)

                        # log_string = record['command_string'] + " PID = " + str(record['process'].pid + 'DIED')
                        logger = logging.getLogger()
                        logger.error(log_string)

                        # reset its state and process, and process ID
                        record['process'] = None
                        record['process_id'] = None

                        # do we need to restart it?
                        if record['auto_restart'] == 'yes':
                            self.spawn_local(x)
        else:
            for x, record in enumerate(self.launch_db):
                if not record['command_string'] == 'backplane':
                    if record['process_id']:
                        pids = psutil.pids()
                        # print(record)
                        # check to see if the process in the launch_db is in this list
                        # status = None

                        try:
                            if record['process'].pid in pids:
                                proc = psutil.Process(record['process'].pid)
                                status = proc.status()
                                # print('command: ' + record['command_string'] + 'status: ' + status)

                                if status == psutil.STATUS_SLEEPING:
                                    continue

                                if not status:
                                    status = psutil.STATUS_ZOMBIE

                                if status == psutil.STATUS_ZOMBIE:

                                    log_string = '{:35} PID = {} DIED'.format(record['command_string'],
                                                                              str(record['process'].pid))
                                    print(log_string)

                                    logger = logging.getLogger()
                                    logger.error(log_string)

                                    # reset its process, and process ID
                                    record['process'] = None
                                    record['process_id'] = None

                                    # do we need to restart it?
                                    if record['auto_restart'] == 'yes':
                                        self.spawn_local(x)
                        except AttributeError:
                            pass

    def incoming_message_processing(self, topic, payload):
        """
        Messages are sent here from the receive_loop

        :param topic: Message Topic string
        :param payload: Message Data
        :return:
        """
        # make sure this is not a duplicate launch request

        if topic == 'killall':
            # self.scheduler.shutdown()
            self.clean_up()
            # sys.exit(0)

        else:
            # check to see if the process was already launched and ignore
            # if it was
            for idx, record in enumerate(self.launch_db):
                if record['launch_id'] == payload['launch_id']:
                    return
            # print(topic, payload)
            # idx = len(self.launch_db)
            self.launch_db.append(payload)
            self.spawn_local(len(self.launch_db) - 1)

            record = self.launch_db[-1]

            # pid = str(record['process_id'])
            # send acknowledgement to server
            ack = {'command_string': record['command_string'],
                   'process_id': record['process_id'],
                   'launch_id': record['launch_id']}
            topic = record['reply_topic']
            self.publish_payload(ack, topic)

    def clean_up(self):
        """
        Graceful shutdown - all newly opened windows and associated processes
        are killed
        :return:
        """
        # self.scheduler.shutdown()
        # self.publish_payload({'kill': True}, 'killall')
        # time.sleep(.5)
        self.scheduler.pause()
        for idx, record in enumerate(self.launch_db):
            if record['process']:
                print('{:35} PID = {} KILLED'.format(record['command_string'], str(record['process'].pid)))
                proc = psutil.Process(record['process'].pid)
                proc.kill()
                record['process'] = None
                record['process_id'] = None
        sys.exit(0)


def blc():
    # allow user to bypass the IP address auto-discovery. This is necessary if the component resides on a computer
    # other than the computing running the backplane.

    parser = argparse.ArgumentParser()

    parser.add_argument("-b", dest="back_plane_ip_address", default="None",
                        help="None or IP address used by Back Plane",
                        required=True)
    parser.add_argument("-p", dest="publisher_port", default='43124',
                        help="Publisher IP port")
    parser.add_argument("-t", dest="topic", default='None',
                        help="Command Receiver Topic", required=True)
    parser.add_argument("-s", dest="subscriber_port", default='43125',
                        help="Subscriber IP port")

    args = parser.parse_args()

    kw_options = {
        'back_plane_ip_address': args.back_plane_ip_address,
        'publisher_port': args.publisher_port,
        'subscriber_port': args.subscriber_port,
        'topic': args.topic
    }

    # replace with the name of your class
    BLC(**kw_options)


# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)


if __name__ == '__main__':
    # replace with name of function you defined above
    blc()



"""
banyan_base.py

 Copyright (c) 2016-2021 Alan Yorinks All right reserved.

 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
from __future__ import unicode_literals

# Use argparse and signal if you wish to implement the argparse
# code located at the bottom of this file.

# import argparse
# import signal
# import sys

import socket
import time
import msgpack
import msgpack_numpy as m
import zmq
import psutil


class BanyanBase(object):
    """

    This is the base class for all Python Banyan components,
    encapsulating and acting as an abstraction layer for zeromq and message pack
    functionality.

    Banyan components are derived by inheriting from this class and
    overriding its methods as necessary.

    Banyan components have the capability to both publish and subscribe to user
    defined messages using the Banyan backplane.

    To import into  the derived class use:

           from python_banyan.banyan_base import BanyanBase

    """

    # noinspection PyBroadException
    def __init__(self, back_plane_ip_address=None, subscriber_port='43125',
                 publisher_port='43124', process_name='None', loop_time=.1, numpy=False,
                 external_message_processor=None, receive_loop_idle_addition=None,
                 connect_time=0.3):
        """
        The __init__ method sets up all the ZeroMQ "plumbing"

        :param back_plane_ip_address: banyan_base back_planeIP Address -
                                      if not specified, it will be set to the
                                      local computer.

        :param subscriber_port: banyan_base back plane subscriber port.
               This must match that of the banyan_base backplane

        :param publisher_port: banyan_base back plane publisher port.
                               This must match that of the banyan_base backplane.

        :param process_name: Component identifier in banner at component startup.

        :param loop_time: Receive loop sleep time.

        :param numpy: Set true if you wish to include numpy matrices in your messages.

        :param external_message_processor: external method to process messages

        :param receive_loop_idle_addition: an external method called in the idle section
                                           of the receive loop

        :param connect_time: a short delay to allow the component to connect
                             to the Backplane
        """

        # call to super allows this class to be used in multiple
        # inheritance scenarios when needed
        super(BanyanBase, self).__init__()

        self.backplane_exists = False

        self.back_plane_ip_address = None
        self.numpy = numpy
        self.external_message_processor = external_message_processor
        self.receive_loop_idle_addition = receive_loop_idle_addition
        self.connect_time = connect_time

        # if using numpy apply the msgpack_numpy monkey patch
        if numpy:
            m.patch()

        # If no back plane address was specified, determine the IP address of the local machine
        if back_plane_ip_address:
            self.back_plane_ip_address = back_plane_ip_address
        else:
            # check for a running backplane
            for pid in psutil.pids():
                try:
                    p = psutil.Process(pid)
                except psutil.NoSuchProcess:
                    # the process list from psutil.pids() may be outdated
                    continue
                try:
                    p_command = p.cmdline()
                except psutil.AccessDenied:
                    # occurs in Windows - ignore
                    continue
                try:
                    if any('backplane' in s for s in p_command):
                        self.backplane_exists = True
                    else:
                        continue
                except UnicodeDecodeError:
                    continue

            if not self.backplane_exists:
                raise RuntimeError('Backplane is not running - please start it.')
            # determine this computer's IP address
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            # use the google dns
            try:
                s.connect(('8.8.8.8', 1))
                self.back_plane_ip_address = s.getsockname()[0]
            except Exception:
                self.back_plane_ip_address = '127.0.0.1'
            finally:
                s.close()

        self.subscriber_port = subscriber_port
        self.publisher_port = publisher_port

        self.loop_time = loop_time

        print('\n************************************************************')
        print(process_name + ' using Back Plane IP address: ' + self.back_plane_ip_address)
        print('Subscriber Port = ' + self.subscriber_port)
        print('Publisher  Port = ' + self.publisher_port)
        print('Loop Time = ' + str(loop_time) + ' seconds')
        print('************************************************************')

        # establish the zeromq sub and pub sockets and connect to the backplane
        self.my_context = zmq.Context()
        self.subscriber = self.my_context.socket(zmq.SUB)
        connect_string = "tcp://" + self.back_plane_ip_address + ':' + self.subscriber_port
        self.subscriber.connect(connect_string)

        self.publisher = self.my_context.socket(zmq.PUB)
        connect_string = "tcp://" + self.back_plane_ip_address + ':' + self.publisher_port
        self.publisher.connect(connect_string)

        # Allow enough time for the TCP connection to the Backplane complete.
        time.sleep(self.connect_time)

    def get_subscriber(self):
        """
        Retrieve the zmq subscriber object

        :return: self.subscriber
        """
        return self.subscriber

    def get_publisher(self):
        """
        Retrieve the zmq subscriber object

        :return: self.subscriber
        """
        return self.publisher

    def set_subscriber_topic(self, topic):
        """
        This method sets a subscriber topic.

        You can subscribe to multiple topics by calling this method for
        each topic.

        :param topic: A topic string
        """

        if not type(topic) is str:
            raise TypeError('Subscriber topic must be python_banyan string')

        self.subscriber.setsockopt(zmq.SUBSCRIBE, topic.encode())

    def publish_payload(self, payload, topic=''):
        """
        This method will publish a python_banyan payload and its associated topic

        :param payload: Protocol message to be published

        :param topic: A string value
        """

        # make sure the topic is a string
        if not type(topic) is str:
            raise TypeError('Publish topic must be python_banyan string', 'topic')

        # create python_banyan message pack payload
        if self.numpy:
            message = msgpack.packb(payload, default=m.encode)
        else:
            message = msgpack.packb(payload, use_bin_type=True)

        pub_envelope = topic.encode()
        self.publisher.send_multipart([pub_envelope, message])

    def receive_loop(self):
        """
        This is the receive loop for Banyan messages.

        This method may be overwritten to meet the needs
        of the application before handling received messages.

        """
        while True:
            try:
                data = self.subscriber.recv_multipart(zmq.NOBLOCK)
                if self.numpy:
                    payload2 = {}
                    payload = msgpack.unpackb(data[1], object_hook=m.decode)
                    # convert keys to strings
                    # this compensates for the breaking change in msgpack-numpy 0.4.1 to 0.4.2
                    for key, value in payload.items():
                        if not type(key) == str:
                            key = key.decode('utf-8')
                            payload2[key] = value

                    if payload2:
                        payload = payload2
                    self.incoming_message_processing(data[0].decode(), payload)
                else:
                    self.incoming_message_processing(data[0].decode(),
                                                     msgpack.unpackb(data[1], raw=False))
            # if no messages are available, zmq throws this exception
            except zmq.error.Again:
                try:
                    if self.receive_loop_idle_addition:
                        self.receive_loop_idle_addition()
                    time.sleep(self.loop_time)
                except KeyboardInterrupt:
                    self.clean_up()
                    raise KeyboardInterrupt

    def incoming_message_processing(self, topic, payload):
        """
        Override this method with a custom Banyan message processor for subscribed messages.

        :param topic: Message Topic string.

        :param payload: Message Data.
        """
        if self.external_message_processor:
            self.external_message_processor(topic, payload)
        else:
            print('this method should be overwritten in the child class', topic, payload)

    def clean_up(self):
        """
        Clean up before exiting - override if additional cleanup is necessary

        """
        self.publisher.close()
        self.subscriber.close()
        self.my_context.term()

# When creating a derived component, replicate the code below and replace
# banyan_base with a name of your choice.

# def banyan_base():
#     # Allow user to bypass the IP address auto-discovery.
#     # This is necessary if the component resides on a computer
#     # other than the computing running the backplane.
#
#     parser = argparse.ArgumentParser()
#     parser.add_argument("-b", dest="back_plane_ip_address", default="None",
#                         help="None or IP address used by Back Plane")
#
#     # allow the user to specify a name for the component and have it shown on the console banner.
#     # modify the default process name to one you wish to see on the banner.
#     # change the default in the derived class to set the name
#     parser.add_argument("-n", dest="process_name", default="YOUR PROCESS NAME", help="Set process name in banner")
#
#     parser.add_argument("-t", dest="loop_time", default=".1", help="Event Loop Timer in seconds")
#
#     args = parser.parse_args()
#     kw_options = {}
#
#     if args.back_plane_ip_address != 'None':
#         kw_options['back_plane_ip_address'] = args.back_plane_ip_address
#
#     kw_options['process_name'] = args.process_name
#
#     kw_options['loop_time'] = float(args.loop_time)
#
#     # replace with the name of your class
#     app = BanyanBase(**kw_options)
#
#     # optionally add any subscriber topics here
#     app.set_subscriber_topic('python_banyan')
#
#     # optionally start the receive loop here or start it in your __init__
#     try:
#         app.receive_loop()
#     except KeyboardInterrupt:
#         sys.exit()
#
# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
# def signal_handler(sig, frame):
#     print('Exiting Through Signal Handler')
#     raise KeyboardInterrupt
#
#
# # listen for SIGINT
# signal.signal(signal.SIGINT, signal_handler)
# signal.signal(signal.SIGTERM, signal_handler)
#
#
# if __name__ == '__main__':
#     # replace with name of function you defined above
#     banyan_base()



import network, rp2, time
import socket
import umsgpack
from machine import Pin

led = Pin("LED", Pin.OUT)
led.on()

# set your WiFi Country
rp2.country('US')

wlan = network.WLAN(network.STA_IF)
wlan.active(True)

# set power mode to get WiFi power-saving off (if needed)
wlan.config(pm = 0xa11140)

wlan.connect('A-Net', 'Sam2Curly')


while not wlan.isconnected() and wlan.status() >= 0:
 print("Waiting to connect:")
 time.sleep(1)

print(wlan.ifconfig())

# create a socket server
serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

addr = ("",31335)
serversocket.bind(addr)
serversocket.listen()
(clientsocket, address) = serversocket.accept()
print((clientsocket, address))
led.off()

count = 0
while True:
    data = clientsocket.recv(1024)
    if data:
        z = umsgpack.loads(data)
        print(z)
    # payload = {'from_the_pico': count}
    # count += 1
    # packed = umsgpack.dumps(payload)
    # clientsocket.sendall(packed)
    # print('sending')






#!/usr/bin/env python3

"""
 Copyright (c) 2018-2019 Alan Yorinks - All Rights Reserved.


 Python Banyan is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import signal
import sys

from python_banyan.banyan_base import BanyanBase


class BLK(BanyanBase):
    """
    This is the banyan launcher killer
    """

    def __init__(self):
        super(BLK, self).__init__(process_name='Launcher Killer')

        self.publish_payload({'exit': 'exit'}, 'killall')

        sys.exit(0)


def blk():
    BLK()


# signal handler function called when Control-C occurs
# noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
def signal_handler(sig, frame):
    print('Exiting Through Signal Handler')
    raise KeyboardInterrupt


# listen for SIGINT
signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)

if __name__ == '__main__':
    # replace with name of function you defined above
    blk()



from setuptools import setup

with open('pypi_desc.md') as f:
    long_description = f.read()

setup(
    name='python-banyan',
    version='3.14',
    packages=[
        'python_banyan',
        'python_banyan.banyan_base',
        'python_banyan.banyan_base_aio',
        'python_banyan.banyan_base_multi',
        'python_banyan.gateway_base',
        'python_banyan.gateway_base_aio',
        'python_banyan.utils',
        'python_banyan.utils.monitor',
        'python_banyan.utils.banyan_launcher',
        'python_banyan.utils.mqtt_gateway',
        'python_banyan.utils.tcp_gateway',
        'python_banyan.backplane',
    ],
    install_requires=[
        'pyzmq',
        'msgpack-python',
        'numpy>=1.9',
        'msgpack-numpy',
        'psutil',
        'apscheduler',
        'websockets'
    ],

    entry_points={
        'console_scripts': [
            'backplane = python_banyan.backplane.backplane:bp',
            'monitor = python_banyan.utils.monitor.monitor:monitor',
            'bls = python_banyan.utils.banyan_launcher.bls:bls',
            'blc = python_banyan.utils.banyan_launcher.blc:blc',
            'blk = python_banyan.utils.banyan_launcher.blk:blk',
            'mgw = python_banyan.utils.mqtt_gateway.mqtt_gateway:mqtt_gateway',
            'tgw = python_banyan.utils.tcp_gateway.tcp_gateway:tcp_gateway',
        ]
    },

    url='https://github.com/MrYsLab/python_banyan',
    license='GNU Affero General Public License v3 or later (AGPLv3+)',
    author='Alan Yorinks',
    author_email='MisterYsLab@gmail.com',
    description='A Non-Blocking Event Driven Applications Framework',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['python banyan',  'RPC', 'Remote Procedure Call', 'Event Driven',
              'Asynchronous', 'Non-Blocking',
              'Raspberry Pi', 'ZeroMQ', 'MessagePack', 'RedBot'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Other Environment',
        'Intended Audience :: Education',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Education',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: System :: Hardware'
    ],
)
